#!/usr/bin/env python

import os, sys, re

class LOGManager():

  def __init__( self , verbose):
    
    self.verbose = verbose

  def add( self, string ):
    
    if self.verbose:
      print(string)

class SRTElement():

  def __init__( self, number, time_str, subtitle_str ):
  
    self.number = number
    self.parse_time( time_str ) # set self.start_time, self.end_time and self.style
    self.subtitle = subtitle_str
  
  # PUBLIC METHOS
  
  def shift( self, milliseconds ):
  
    self.start_time.shift(milliseconds)
    self.end_time.shift(milliseconds)
  
  def change_time( self, new_time ):
  
    shift = new_time - self.start_time.milliseconds()
    self.shift( shift )
  
  def extend( self, milliseconds ):
  
    self.end_time.shift(milliseconds)
  
  def string( self ):
    
    time_string = self.start_time.string() + " --> " + self.end_time.string()
    if self.style != None:
      time_string = time_string + " " + self.style
  
    complete_string = str(self.number) + "\n" + time_string + "\n" + self.subtitle
    
    return complete_string
  
  # PRIVATE METHODS
  
  def parse_time( self, time_str ):
    
    time_exp = re.compile( '^([0-9]{2})\:([0-9]{2})\:([0-9]{2}),([0-9]{3})\s?-->\s?([0-9]{2})\:([0-9]{2})\:([0-9]{2}),([0-9]{3})\s?(X1\:[0-9]*\sX2\:[0-9]*\sY1\:[0-9]*\sY2\:[0-9]*)?' )
    
    res=time_exp.match( time_str )
    if res != None:
      hh = res.group(1)
      mm = res.group(2)
      ss = res.group(3)
      ml = res.group(4)
      self.start_time = SRTTime( hh, mm, ss, ml )
      hh = res.group(5)
      mm = res.group(6)
      ss = res.group(7)
      ml = res.group(8)
      self.end_time = SRTTime( hh, mm, ss, ml )
      self.style = res.group(9)
    else:
      print("Error, string " + time_str + " is not a valid time string ")

class SRTTime():

  def __init__(self, hh, mm, ss, ml):
    
    self.abs_time = int(hh) * 3600000 + int(mm) * 60000 + int(ss) * 1000 + int(ml) ## FIXME
  
  # PUBLIC METHODS
  
  def shift( self, milliseconds ):
  
    self.abs_time += int( milliseconds )
    if self.abs_time < 0:
      self.abs_time = 0
    
    return self
  
  def string( self ):
  
    hh = int( self.abs_time / 3600000 )
    rem = self.abs_time % 3600000
    mm = int( rem / 60000 )
    rem = rem % 60000
    ss = int( rem / 1000 )
    ml = rem % 1000
    
    hh = ("0" + str(hh) )[-2:]
    mm = ("0" + str(mm) )[-2:]
    ss = ("0" + str(ss) )[-2:]
    ml = ("00" + str(ml) )[-3:]
    
    string = hh + ":" + mm + ":" + ss + "," + ml
    
    return string
  
  def milliseconds( self ):
  
    return self.abs_time

class SRTManager():
  
  def __init__( self, srt_input, verbose ):
  
    self.verbose = verbose
    self.srt_input = srt_input
    self.srt_output = srt_input.replace('.srt', '_new.srt', 1)
    
    self.log_manager = LOGManager( self.verbose )
    
    self.srt_elements = self.parse( )

  ## PUBLIC METHODS
  
  def shift( self, milliseconds ):
    
    if self.verbose:
      sec = int( int( milliseconds ) / 1000 )
      print( "Shifting of " + str(sec) + " seconds " )
    
    for element in self.srt_elements:
      element.shift(milliseconds)
  
  def extend( self, milliseconds ):
  
    if self.verbose:
      sec = int( int ( milliseconds ) / 1000 )
      print( "Extending of " + str(sec) + " seconds " )
    
    for element in self.srt_elements:
      element.extend(milliseconds)
  
  def align( self, align_str ):
  
    if self.verbose:
      print( "SRTManager == align :: Aligning subs" )
  
    tmp = align_str.split('+')
    if len(tmp) != 2:
      print("Wrong ALIGN string format")
      sys.exit(1)
    align_re = re.compile('([0-9]{2})\:([0-9]{2})\:([0-9]{2}),([0-9]{3})-->([0-9]{2})\:([0-9]{2})\:([0-9]{2}),([0-9]{3})')
    # FTS time
    res = align_re.match(tmp[0])
    if res == None:
      print("Wrong ALIGN string format")
      sys.exit(1)
    else:
      hh = res.group(1)
      mm = res.group(2)
      ss = res.group(3)
      ml = res.group(4)
      fst_sub_time = SRTTime( hh, mm, ss, ml )
      hh = res.group(5)
      mm = res.group(6)
      ss = res.group(7)
      ml = res.group(8)
      fst_film_time = SRTTime( hh, mm, ss, ml )
    # SND time
    res = align_re.match(tmp[1])
    if res == None:
      print("Wrong ALIGN string format")
      sys.exit(1)
    else:
      hh = res.group(1)
      mm = res.group(2)
      ss = res.group(3)
      ml = res.group(4)
      snd_sub_time = SRTTime( hh, mm, ss, ml )
      hh = res.group(5)
      mm = res.group(6)
      ss = res.group(7)
      ml = res.group(8)
      snd_film_time = SRTTime( hh, mm, ss, ml )
    
    # FST align
    shift = fst_film_time.milliseconds() - fst_sub_time.milliseconds()
    self.shift( shift )
    
    # SND align
    # now I have the 1st subtitle aligned with the video
    # I alling the second then
    # f(x)=x0+(x-x0)*(y1-x0)/(y0-x0)
    # x0=fst_time[1]
    # y0=snd_time[0]+shift_time
    # y1=snd_time[1]
    
    constant = ( snd_film_time.milliseconds() - fst_film_time.milliseconds() ) / ( snd_sub_time.milliseconds() - fst_film_time.milliseconds() + shift )
    
    if self.verbose:
      print("Alignment constant: " + str(constant) )
    
    for element in self.srt_elements:
      if element.start_time.milliseconds() >= fst_film_time.milliseconds():
        new_position = int( fst_film_time.milliseconds() + ( element.start_time.milliseconds() - fst_film_time.milliseconds() ) * constant )
        element.change_time( new_position )
  
  def write_new_srt_file( self ):
  
    if self.verbose:
      print( "SRTManager == write_new_srt_file :: Writing output to " + self.srt_output )
  
    hand = open( self.srt_output, "w" )
    for element in self.srt_elements:
      hand.write( element.string() + "\n\n" )
  
    hand.close()
    
  ## PRIVATE METHODS
  
  def parse( self ):
  
    if self.verbose:
      print( "SRTManager == parse :: Parsing file " + self.srt_input )
    
    if os.path.exists( self.srt_input ):
      hand = open( self.srt_input, "r" )
      cont = True
      number = 0
      time_str = None
      subtitle = ''
      current_element = None
      
      next_is_time = False
      srt_elements = []
      
      for line in hand:
        line = line.rstrip()
        if line == '':
          # EMPTY LINE -> new element
          if number != 0:
            subtitle = subtitle.rstrip()
            current_element = SRTElement( number, time_str, subtitle )
            srt_elements.append( current_element )
          number = 0
          time_str = None
          subtitle = ''
        elif line.isnumeric() and number == 0:
          # SUBTITLE NUMBER
          number = line
          next_is_time = True
        elif next_is_time:
          # TIME STRING
          next_is_time = False
          time_str = line
        else:
          # SUBTITLE
          subtitle = subtitle + line + "\n"
      if number != 0:
        subtitle = subtitle.rstrip()
        current_element = SRTElement( number, time_str, subtitle )
        srt_elements.append( current_element )
      hand.close()
      
      return srt_elements
      
    else:
      print( "Error, file " + self.srt_input + " not found!" )
      sys.exit(1)
