#!/usr/bin/env python3

import sys, argparse
from lib.SRTManager import SRTManager

# get the options from the command line
parser = argparse.ArgumentParser( description = "Change the timing of the subtitles in the given .srt file" )
parser.add_argument( 'srt_file', metavar = 'SRT_FILEPATH', help = '.srt file' )
parser.add_argument( '--shift', '-s', metavar = 'INTEGER', default = '0', help = 'Shift the subtitles of INTEGER milliseconds' )
parser.add_argument( '--extend', '-e', metavar = 'INTEGER', default = '0', help = 'Extend the duration of the subtitles of INTEGER milliseconds' )
parser.add_argument( '--align', metavar = 'TIMESUB_A-->TIMEFILM_A+TIMESUB_B-->TIMEFILM_B', default = 'none', help = '''Rewrite the srt file such that the subtitle at time [TIMESUB_A] appears at time [TIMEFILM_A], the same for [TIMESUB_B].
The other subs are shifted correspondingly.
The arguments TIMESUB_A, TIMESUB_B, TIMEFILM_A, TIMEFILM_B must be in the format hh:mm:ss,lll (hour,min,sec,milliseconds)''' )
parser.add_argument( '--verbose', '-v', action = 'store_true', help = 'Verbose output' )
args = parser.parse_args()

srt_file = args.srt_file
shift = args.shift
extend = args.extend
align = args.align
verbose = args.verbose

if align != "none" and shift != "0":
  print("Error: The shift and align operations are incompatible!")
  sys.exit(0)

manager = SRTManager( srt_file, verbose )
if shift != '0':
  manager.shift( shift )
if extend != '0':
  manager.extend( extend )
if align != 'none':
  manager.align( align )
manager.write_new_srt_file()

sys.exit(0)
